# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.SQLAdmin.V1.Model.Settings do
  @moduledoc """
  Database instance settings.

  ## Attributes

  *   `advancedMachineFeatures` (*type:* `GoogleApi.SQLAdmin.V1.Model.AdvancedMachineFeatures.t`, *default:* `nil`) - Specifies advance machine configuration for the instance relevant only for SQL Server.
  *   `collation` (*type:* `String.t`, *default:* `nil`) - The name of server Instance collation.
  *   `dataCacheConfig` (*type:* `GoogleApi.SQLAdmin.V1.Model.DataCacheConfig.t`, *default:* `nil`) - Configuration for data cache.
  *   `availabilityType` (*type:* `String.t`, *default:* `nil`) - Availability type. Potential values: * `ZONAL`: The instance serves data from only one zone. Outages in that zone affect data accessibility. * `REGIONAL`: The instance can serve data from more than one zone in a region (it is highly available)./ For more information, see [Overview of the High Availability Configuration](https://cloud.google.com/sql/docs/mysql/high-availability).
  *   `userLabels` (*type:* `map()`, *default:* `nil`) - User-provided labels, represented as a dictionary where each label is a single key value pair.
  *   `insightsConfig` (*type:* `GoogleApi.SQLAdmin.V1.Model.InsightsConfig.t`, *default:* `nil`) - Insights configuration, for now relevant only for Postgres.
  *   `ipConfiguration` (*type:* `GoogleApi.SQLAdmin.V1.Model.IpConfiguration.t`, *default:* `nil`) - The settings for IP Management. This allows to enable or disable the instance IP and manage which external networks can connect to the instance. The IPv4 address cannot be disabled for Second Generation instances.
  *   `dataDiskSizeGb` (*type:* `String.t`, *default:* `nil`) - The size of data disk, in GB. The data disk size minimum is 10GB.
  *   `timeZone` (*type:* `String.t`, *default:* `nil`) - Server timezone, relevant only for Cloud SQL for SQL Server.
  *   `locationPreference` (*type:* `GoogleApi.SQLAdmin.V1.Model.LocationPreference.t`, *default:* `nil`) - The location preference settings. This allows the instance to be located as near as possible to either an App Engine app or Compute Engine zone for better performance. App Engine co-location was only applicable to First Generation instances.
  *   `kind` (*type:* `String.t`, *default:* `nil`) - This is always `sql#settings`.
  *   `backupConfiguration` (*type:* `GoogleApi.SQLAdmin.V1.Model.BackupConfiguration.t`, *default:* `nil`) - The daily backup configuration for the instance.
  *   `crashSafeReplicationEnabled` (*type:* `boolean()`, *default:* `nil`) - Configuration specific to read replica instances. Indicates whether database flags for crash-safe replication are enabled. This property was only applicable to First Generation instances.
  *   `replicationType` (*type:* `String.t`, *default:* `nil`) - The type of replication this instance uses. This can be either `ASYNCHRONOUS` or `SYNCHRONOUS`. (Deprecated) This property was only applicable to First Generation instances.
  *   `activeDirectoryConfig` (*type:* `GoogleApi.SQLAdmin.V1.Model.SqlActiveDirectoryConfig.t`, *default:* `nil`) - Active Directory configuration, relevant only for Cloud SQL for SQL Server.
  *   `authorizedGaeApplications` (*type:* `list(String.t)`, *default:* `nil`) - The App Engine app IDs that can access this instance. (Deprecated) Applied to First Generation instances only.
  *   `edition` (*type:* `String.t`, *default:* `nil`) - Optional. The edition of the instance.
  *   `databaseReplicationEnabled` (*type:* `boolean()`, *default:* `nil`) - Configuration specific to read replica instances. Indicates whether replication is enabled or not. WARNING: Changing this restarts the instance.
  *   `sqlServerAuditConfig` (*type:* `GoogleApi.SQLAdmin.V1.Model.SqlServerAuditConfig.t`, *default:* `nil`) - SQL Server specific audit configuration.
  *   `activationPolicy` (*type:* `String.t`, *default:* `nil`) - The activation policy specifies when the instance is activated; it is applicable only when the instance state is RUNNABLE. Valid values: * `ALWAYS`: The instance is on, and remains so even in the absence of connection requests. * `NEVER`: The instance is off; it is not activated, even if a connection request arrives.
  *   `passwordValidationPolicy` (*type:* `GoogleApi.SQLAdmin.V1.Model.PasswordValidationPolicy.t`, *default:* `nil`) - The local user password validation policy of the instance.
  *   `databaseFlags` (*type:* `list(GoogleApi.SQLAdmin.V1.Model.DatabaseFlags.t)`, *default:* `nil`) - The database flags passed to the instance at startup.
  *   `maintenanceWindow` (*type:* `GoogleApi.SQLAdmin.V1.Model.MaintenanceWindow.t`, *default:* `nil`) - The maintenance window for this instance. This specifies when the instance can be restarted for maintenance purposes.
  *   `dataDiskType` (*type:* `String.t`, *default:* `nil`) - The type of data disk: `PD_SSD` (default) or `PD_HDD`. Not used for First Generation instances.
  *   `storageAutoResizeLimit` (*type:* `String.t`, *default:* `nil`) - The maximum size to which storage capacity can be automatically increased. The default value is 0, which specifies that there is no limit.
  *   `denyMaintenancePeriods` (*type:* `list(GoogleApi.SQLAdmin.V1.Model.DenyMaintenancePeriod.t)`, *default:* `nil`) - Deny maintenance periods
  *   `connectorEnforcement` (*type:* `String.t`, *default:* `nil`) - Specifies if connections must use Cloud SQL connectors. Option values include the following: `NOT_REQUIRED` (Cloud SQL instances can be connected without Cloud SQL Connectors) and `REQUIRED` (Only allow connections that use Cloud SQL Connectors). Note that using REQUIRED disables all existing authorized networks. If this field is not specified when creating a new instance, NOT_REQUIRED is used. If this field is not specified when patching or updating an existing instance, it is left unchanged in the instance.
  *   `deletionProtectionEnabled` (*type:* `boolean()`, *default:* `nil`) - Configuration to protect against accidental instance deletion.
  *   `pricingPlan` (*type:* `String.t`, *default:* `nil`) - The pricing plan for this instance. This can be either `PER_USE` or `PACKAGE`. Only `PER_USE` is supported for Second Generation instances.
  *   `tier` (*type:* `String.t`, *default:* `nil`) - The tier (or machine type) for this instance, for example `db-custom-1-3840`. WARNING: Changing this restarts the instance.
  *   `enableGoogleMlIntegration` (*type:* `boolean()`, *default:* `nil`) - Optional. Configuration to enable Cloud SQL Vertex AI Integration
  *   `storageAutoResize` (*type:* `boolean()`, *default:* `nil`) - Configuration to increase storage size automatically. The default value is true.
  *   `settingsVersion` (*type:* `String.t`, *default:* `nil`) - The version of instance settings. This is a required field for update method to make sure concurrent updates are handled properly. During update, use the most recent settingsVersion value for this instance and do not try to update this value.
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :advancedMachineFeatures =>
            GoogleApi.SQLAdmin.V1.Model.AdvancedMachineFeatures.t() | nil,
          :collation => String.t() | nil,
          :dataCacheConfig => GoogleApi.SQLAdmin.V1.Model.DataCacheConfig.t() | nil,
          :availabilityType => String.t() | nil,
          :userLabels => map() | nil,
          :insightsConfig => GoogleApi.SQLAdmin.V1.Model.InsightsConfig.t() | nil,
          :ipConfiguration => GoogleApi.SQLAdmin.V1.Model.IpConfiguration.t() | nil,
          :dataDiskSizeGb => String.t() | nil,
          :timeZone => String.t() | nil,
          :locationPreference => GoogleApi.SQLAdmin.V1.Model.LocationPreference.t() | nil,
          :kind => String.t() | nil,
          :backupConfiguration => GoogleApi.SQLAdmin.V1.Model.BackupConfiguration.t() | nil,
          :crashSafeReplicationEnabled => boolean() | nil,
          :replicationType => String.t() | nil,
          :activeDirectoryConfig =>
            GoogleApi.SQLAdmin.V1.Model.SqlActiveDirectoryConfig.t() | nil,
          :authorizedGaeApplications => list(String.t()) | nil,
          :edition => String.t() | nil,
          :databaseReplicationEnabled => boolean() | nil,
          :sqlServerAuditConfig => GoogleApi.SQLAdmin.V1.Model.SqlServerAuditConfig.t() | nil,
          :activationPolicy => String.t() | nil,
          :passwordValidationPolicy =>
            GoogleApi.SQLAdmin.V1.Model.PasswordValidationPolicy.t() | nil,
          :databaseFlags => list(GoogleApi.SQLAdmin.V1.Model.DatabaseFlags.t()) | nil,
          :maintenanceWindow => GoogleApi.SQLAdmin.V1.Model.MaintenanceWindow.t() | nil,
          :dataDiskType => String.t() | nil,
          :storageAutoResizeLimit => String.t() | nil,
          :denyMaintenancePeriods =>
            list(GoogleApi.SQLAdmin.V1.Model.DenyMaintenancePeriod.t()) | nil,
          :connectorEnforcement => String.t() | nil,
          :deletionProtectionEnabled => boolean() | nil,
          :pricingPlan => String.t() | nil,
          :tier => String.t() | nil,
          :enableGoogleMlIntegration => boolean() | nil,
          :storageAutoResize => boolean() | nil,
          :settingsVersion => String.t() | nil
        }

  field(:advancedMachineFeatures, as: GoogleApi.SQLAdmin.V1.Model.AdvancedMachineFeatures)
  field(:collation)
  field(:dataCacheConfig, as: GoogleApi.SQLAdmin.V1.Model.DataCacheConfig)
  field(:availabilityType)
  field(:userLabels, type: :map)
  field(:insightsConfig, as: GoogleApi.SQLAdmin.V1.Model.InsightsConfig)
  field(:ipConfiguration, as: GoogleApi.SQLAdmin.V1.Model.IpConfiguration)
  field(:dataDiskSizeGb)
  field(:timeZone)
  field(:locationPreference, as: GoogleApi.SQLAdmin.V1.Model.LocationPreference)
  field(:kind)
  field(:backupConfiguration, as: GoogleApi.SQLAdmin.V1.Model.BackupConfiguration)
  field(:crashSafeReplicationEnabled)
  field(:replicationType)
  field(:activeDirectoryConfig, as: GoogleApi.SQLAdmin.V1.Model.SqlActiveDirectoryConfig)
  field(:authorizedGaeApplications, type: :list)
  field(:edition)
  field(:databaseReplicationEnabled)
  field(:sqlServerAuditConfig, as: GoogleApi.SQLAdmin.V1.Model.SqlServerAuditConfig)
  field(:activationPolicy)
  field(:passwordValidationPolicy, as: GoogleApi.SQLAdmin.V1.Model.PasswordValidationPolicy)
  field(:databaseFlags, as: GoogleApi.SQLAdmin.V1.Model.DatabaseFlags, type: :list)
  field(:maintenanceWindow, as: GoogleApi.SQLAdmin.V1.Model.MaintenanceWindow)
  field(:dataDiskType)
  field(:storageAutoResizeLimit)

  field(:denyMaintenancePeriods,
    as: GoogleApi.SQLAdmin.V1.Model.DenyMaintenancePeriod,
    type: :list
  )

  field(:connectorEnforcement)
  field(:deletionProtectionEnabled)
  field(:pricingPlan)
  field(:tier)
  field(:enableGoogleMlIntegration)
  field(:storageAutoResize)
  field(:settingsVersion)
end

defimpl Poison.Decoder, for: GoogleApi.SQLAdmin.V1.Model.Settings do
  def decode(value, options) do
    GoogleApi.SQLAdmin.V1.Model.Settings.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.SQLAdmin.V1.Model.Settings do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
